/*
 *   Copyright (C) 2020 CERN.
 *   Copyright (C) 2020 Northwestern University.
 *   Copyright (C) 2021 Graz University of Technology.
 *   Copyright (C) 2021-2022 New York University.
 *
 * Invenio RDM Records is free software; you can redistribute it and/or modify
 * it under the terms of the MIT License; see LICENSE file for more details.
 */

.creatibutors,
.ui.items > .item .meta .creatibutors {
  margin: 0 0 0.5rem 0;
  display: flex;
  flex-wrap: wrap;

  &:last-child {
    margin-bottom: 0;
  }

  dt {
    font-weight: bold;
    margin: 0 0.25rem 0 0;

    &.hidden {
      display: none;
    }
  }

  .creatibutor-wrap {
    color: @mutedTextColor;
    display: flex;
    flex-wrap: wrap;

    &:nth-child(n):not(:last-child) {
      margin-right: 1rem;
    }

    .identifier-link, .group.icon {
      margin: 0 .1rem 0 .2rem;
    }
  }
}
